@@ -1,3 +1,5 @@
+{-# LANGUAGE DeriveAnyClass #-}
+{-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE Rank2Types, CPP, FlexibleContexts #-}
 module Categorize where
 
@@ -6,15 +8,13 @@ import Data
 import qualified Text.Regex.PCRE.Light.Text as RE
 import qualified Data.MyText as T
 import Data.MyText (Text)
-import Control.Applicative (empty, (<*))
+import Control.Applicative (empty, (<*), (<$), (<$>))
 import Control.Monad
-import Control.Monad.Instances()
 import Control.Monad.Trans.Reader hiding (local)
 import Control.Monad.Reader.Class (local)
 import Control.Monad.Trans.Class
 import Data.Functor.Identity
 
-import Control.Applicative ((<$>))
 import Control.DeepSeq
 import Data.Char
 import Data.List
@@ -39,8 +39,10 @@ import System.Locale (defaultTimeLocale, iso8601DateFormat)
 #endif
 import Debug.Trace
 import Text.Printf
+import GHC.Generics (Generic)
 
 type Categorizer = TimeLog CaptureData -> TimeLog (Ctx, ActivityData)
+type ApplyCond = TimeLogEntry (Ctx, ActivityData) -> Bool
 type Rule = Ctx -> ActivityData
 type Environment = Map String Cond
 
@@ -48,16 +50,12 @@ type Parser = ParsecT String () (ReaderT (TimeZone, Environment) Identity)
 
 data Ctx = Ctx
         { cNow :: TimeLogEntry CaptureData
-        , cCurrentWindow :: Maybe (Bool, Text, Text)
-        , cWindowInScope :: Maybe (Bool, Text, Text)
+        , cCurrentWindow :: Maybe WindowData
+        , cWindowInScope :: Maybe WindowData
         , cSubsts :: [Text]
-        , cCurrentTime :: UTCTime
-        , cTimeZone :: TimeZone
+        , cCurrentTime :: ZonedTime
         , conditionBindings :: Map String Cond
-        } deriving Show
-
-instance NFData Ctx where
-    rnf (Ctx a b c d e f g) = a `deepseq` b `deepseq` c `deepseq` d `deepseq` e `deepseq` f `deepseq` g `deepseq` ()
+        } deriving (Show, Generic, NFData)
 
 type Cond = CtxFun [Text]
 
@@ -94,25 +92,28 @@ readCategorizer :: FilePath -> IO Categorizer
 readCategorizer filename = withFile filename ReadMode $ \h -> do
         hSetEncoding h utf8
         content <- hGetContents h
-        time <- getCurrentTime
-        tz <- getCurrentTimeZone
-        case runParserStack (tz, Map.empty) (between (return ()) eof parseRules) filename content of
+        time <- getZonedTime
+        case runParserStack (zonedTimeZone time, Map.empty) (between (return ()) eof parseRules) filename content of
           Left err -> do
                 putStrLn "Parser error:"
                 print err
                 exitFailure
           Right cat -> return
-                (map (fmap (mkSecond (postpare . cat))) . prepare time tz)
+                (map (fmap (mkSecond (postpare . cat))) . prepare time)
 
-applyCond :: String -> TimeZone -> Environment -> TimeLogEntry (Ctx, ActivityData) -> Bool
-applyCond s tz env =
-        case runParserStack (tz, env) (parseCond <* eof) "command line parameter" s of
-          Left err -> error (show err)
-          Right c  -> isJust . c . fst . tlData
+mkApplyCond :: String -> IO ApplyCond
+mkApplyCond s = do
+        tz <- getCurrentTimeZone
+        case runParserStack (tz, Map.empty) (parseCond <* eof) "command line parameter" s of
+          Left err -> do
+                putStrLn "Parser error:"
+                print err
+                exitFailure
+          Right c -> return (isJust . c . fst . tlData)
 
-prepare :: UTCTime -> TimeZone -> TimeLog CaptureData -> TimeLog Ctx
-prepare time tz = map go
-  where go now  = now {tlData = Ctx now (findActive (cWindows (tlData now))) Nothing [] time tz Map.empty }
+prepare :: ZonedTime -> TimeLog CaptureData -> TimeLog Ctx
+prepare time = map go
+  where go now  = now {tlData = Ctx now (find wActive (cWindows (tlData now))) Nothing [] time Map.empty }
 
 -- | Here, we filter out tags appearing twice, and make sure that only one of
 --   each category survives
@@ -134,7 +135,9 @@ lang = makeTokenParser LanguageDef
                 , reservedOpNames= []
                 , reservedNames  = [ "title"
                                    , "program"
+                                   , "wdesktop"
                                    , "active"
+                                   , "hidden"
                                    , "idle"
                                    , "time"
                                    , "sampleage"
@@ -338,7 +341,7 @@ trd3 (_,_,c) = c
 -- Day of week is an integer in [1..7].
 evalDayOfWeek :: CondPrim -> Erring CondPrim
 evalDayOfWeek (CondDate df) = Right $ CondInteger $ \ctx ->
-  let tz = cTimeZone ctx in
+  let tz = zonedTimeZone (cCurrentTime ctx) in
   (toInteger . trd3 . toWeekDate . localDay . utcToLocalTime tz) `fmap` df ctx
 evalDayOfWeek cp = Left $ printf
   "Cannot apply day of week to an expression of type %s, only to $date."
@@ -347,7 +350,7 @@ evalDayOfWeek cp = Left $ printf
 -- Day of month is an integer in [1..31].
 evalDayOfMonth :: CondPrim -> Erring CondPrim
 evalDayOfMonth (CondDate df) = Right $ CondInteger $ \ctx ->
-  let tz = cTimeZone ctx in
+  let tz = zonedTimeZone (cCurrentTime ctx) in
   (toInteger . trd3 . toGregorian . localDay . utcToLocalTime tz) `fmap` df ctx
 evalDayOfMonth cp = Left $ printf
   "Cannot apply day of month to an expression of type %s, only to $date."
@@ -356,7 +359,7 @@ evalDayOfMonth cp = Left $ printf
 -- Month is an integer in [1..12].
 evalMonth :: CondPrim -> Erring CondPrim
 evalMonth (CondDate df) = Right $ CondInteger $ \ctx ->
-  let tz = cTimeZone ctx in
+  let tz = zonedTimeZone (cCurrentTime ctx) in
   (toInteger . snd3 . toGregorian . localDay . utcToLocalTime tz) `fmap` df ctx
 evalMonth cp = Left $ printf
   "Cannot apply month to an expression of type %s, only to $date."
@@ -364,7 +367,7 @@ evalMonth cp = Left $ printf
 
 evalYear :: CondPrim -> Erring CondPrim
 evalYear (CondDate df) = Right $ CondInteger $ \ctx ->
-  let tz = cTimeZone ctx in
+  let tz = zonedTimeZone (cCurrentTime ctx) in
   (fst3 . toGregorian . localDay . utcToLocalTime tz) `fmap` df ctx
 evalYear cp = Left $ printf
   "Cannot apply year to an expression of type %s, only to $date."
@@ -373,7 +376,7 @@ evalYear cp = Left $ printf
 -- format date according to ISO 8601 (YYYY-MM-DD)
 formatDate :: CondPrim -> Erring CondPrim
 formatDate (CondDate df) = Right $ CondString $ \ctx ->
-  let tz = cTimeZone ctx
+  let tz = zonedTimeZone (cCurrentTime ctx)
       local = utcToLocalTime tz `fmap` df ctx
    in T.pack . formatTime defaultTimeLocale (iso8601DateFormat Nothing) <$> local
 formatDate cp = Left $ printf
@@ -405,7 +408,9 @@ parseCondPrim = choice
                   return $ CondString (getBackref backref)
              , choice [ reserved lang "title" >> return (CondString (getVar "title"))
                       , reserved lang "program" >> return (CondString (getVar "program"))
+                      , reserved lang "wdesktop" >> return (CondString (getVar "wdesktop"))
                       , reserved lang "active" >> return (CondCond checkActive)
+                      , reserved lang "hidden" >> return (CondCond checkHidden)
                       , reserved lang "idle" >> return (CondInteger (getNumVar NvIdle))
                       , reserved lang "time" >> return (CondTime (getTimeVar TvTime))
                       , reserved lang "sampleage" >> return (CondTime (getTimeVar TvSampleAge))
@@ -526,12 +531,9 @@ getVar v ctx | "current" `isPrefixOf` v = do
                 let var = drop (length "current.") v
                 win <- cCurrentWindow ctx
                 getVar var (ctx { cWindowInScope = Just win })
-getVar "title"   ctx = do
-                (_,t,_) <- cWindowInScope ctx
-                return t
-getVar "program" ctx = do
-                (_,_,p) <- cWindowInScope ctx
-                return p
+getVar "title"   ctx = wTitle <$> cWindowInScope ctx
+getVar "program" ctx = wProgram <$> cWindowInScope ctx
+getVar "wdesktop" ctx = wDesktop <$> cWindowInScope ctx
 getVar "desktop" ctx = return $ cDesktop (tlData (cNow ctx))
 getVar v _ = error $ "Unknown variable " ++ v
 
@@ -541,23 +543,19 @@ getNumVar NvIdle ctx = Just $ cLastActivity (tlData (cNow ctx)) `div` 1000
 getTimeVar :: TimeVar -> CtxFun NominalDiffTime
 getTimeVar TvTime ctx = Just $
    let utc = tlTime . cNow $ ctx
-       tz = cTimeZone ctx
+       tz = zonedTimeZone (cCurrentTime ctx)
        local = utcToLocalTime tz utc
        midnightUTC = localTimeToUTC tz $ local { localTimeOfDay = midnight }
     in utc `diffUTCTime` midnightUTC
-getTimeVar TvSampleAge ctx = Just $ cCurrentTime ctx `diffUTCTime` tlTime (cNow ctx)
+getTimeVar TvSampleAge ctx = Just $ zonedTimeToUTC (cCurrentTime ctx) `diffUTCTime` tlTime (cNow ctx)
 
 getDateVar :: DateVar -> CtxFun UTCTime
 getDateVar DvDate = Just . tlTime . cNow
-getDateVar DvNow = Just . cCurrentTime
+getDateVar DvNow = Just . zonedTimeToUTC . cCurrentTime
 
-findActive :: [(Bool, t, t1)] -> Maybe (Bool, t, t1)
-findActive = find (\(a,_,_) -> a)
-
-checkActive :: Cond
-checkActive ctx = do (a,_,_) <- cWindowInScope ctx
-                     guard a
-                     return []
+checkActive, checkHidden :: Cond
+checkActive ctx = [] <$ (guard =<< wActive <$> cWindowInScope ctx)
+checkHidden ctx = [] <$ (guard =<< wHidden <$> cWindowInScope ctx)
 
 matchNone :: Rule
 matchNone = const []
