@@ -10,7 +10,7 @@ module Stats (
     defaultFilter,
     defaultReportOptions,
     parseActivityMatcher,
-    filterPredicate,
+    mkFilterPredicate,
     prepareCalculations,
     processReport,
     processRepeater,
@@ -32,6 +32,7 @@ import Data.Time.Format(defaultTimeLocale)
 import System.Locale (defaultTimeLocale)
 #endif
 import Control.Applicative
+import Control.Monad (forM)
 import Data.Strict ((:!:), Pair(..))
 import qualified Data.Strict as Strict
 import Data.Traversable (sequenceA)
@@ -94,15 +95,27 @@ data ReportResults =
         | ListOfIntervals String [Interval]
         | MultipleReportResults [ReportResults]
         | RepeatedReportResults String [(String, ReportResults)]
-        | DumpResult (TimeLog (CaptureData, TimeZone, ActivityData))
+        | DumpResult (TimeLog (CaptureData, ActivityData))
 
 
-filterPredicate :: [Filter] -> TimeLogEntry (Ctx, ActivityData) -> Bool
-filterPredicate filters tl =
-       all (\flag -> case flag of
-                Exclude act  -> excludeTag act tl
-                Only act     -> onlyTag act tl
-                GeneralCond s-> applyCond s (cTimeZone (fst (tlData tl))) M.empty tl) filters
+mkFilterPredicate :: [Filter] -> IO ApplyCond
+mkFilterPredicate filters = do
+    fs <- forM (fastFiltersHeuristic filters) $ \flag -> case flag of
+                Exclude act  -> return $ excludeTag act
+                Only act     -> return $ onlyTag act
+                GeneralCond s-> mkApplyCond s
+    return $ \tl -> all ($ tl) fs
+
+-- | Reorder filters to first apply the presumably cheaper expressions
+-- (e.g. date or sampleage comparisons) given on command-line, hoping that
+-- will filter most samples out and we won't need to run the full categorizer
+-- on these (which we need for 'Exclude'/'Only').
+fastFiltersHeuristic :: [Filter] -> [Filter]
+fastFiltersHeuristic filters = fastFilters ++ slowFilters
+  where
+    (fastFilters, slowFilters) = partition isFast filters
+    isFast (GeneralCond _) = True
+    isFast _ = False
 
 filterActivity :: [ActivityFilter] -> ActivityData -> ActivityData
 filterActivity fs = filter (applyActivityFilter fs)
@@ -274,7 +287,7 @@ processReport tz opts (IntervalTag tag) =
 
 processReport tz opts DumpSamples =
     DumpResult <$> onSelected (mapElems toList $ fmap $
-        \(cd,ad) -> (tlData (cNow cd), cTimeZone cd, filterActivity (roActivityFilter opts) ad)
+        \(cd,ad) -> (tlData (cNow cd), filterActivity (roActivityFilter opts) ad)
         )
 
 calcCategories :: LeftFold (TimeLogEntry (Ctx, ActivityData)) [Category]
